"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.findPkgs = exports.findPkgByName = exports.findPkg = void 0;
const escalade_1 = __importDefault(require("escalade"));
const fs_1 = require("fs");
const module_1 = require("module");
const path_1 = __importDefault(require("path"));
const findPkg = (file) => {
    return (0, escalade_1.default)(file, (_, names) => {
        if (names.includes('package.json')) {
            return 'package.json';
        }
    });
};
exports.findPkg = findPkg;
const readJson = async (file) => {
    const content = await fs_1.promises.readFile(file, 'utf-8');
    return JSON.parse(content);
};
const pkgName = 'package.json';
const findPkgByName = (file, moduleName, callback) => {
    return (0, escalade_1.default)(file, async (dir, names) => {
        if (names.includes(pkgName)) {
            const file = path_1.default.join(dir, pkgName);
            const json = (await readJson(file));
            callback === null || callback === void 0 ? void 0 : callback(file, json);
            if (json.name === moduleName) {
                return pkgName;
            }
        }
    });
};
exports.findPkgByName = findPkgByName;
const findPkgs = async (modulePath, baseDir) => {
    const contextRequire = (0, module_1.createRequire)(path_1.default.resolve(baseDir, '<import>.js'));
    // ensure resolvable
    const moduleFile = contextRequire.resolve(modulePath);
    const isScoped = modulePath[0] === '@';
    const [p1, p2] = isScoped
        ? modulePath.split('/')
        : [modulePath.split('/')[0], ''];
    const moduleName = p1 + (isScoped ? '/' : '') + p2;
    let rootPkg;
    let rootPkgFile;
    try {
        rootPkgFile = `${moduleName}/package.json`;
        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
        rootPkg = require(rootPkgFile);
    }
    catch (_) {
        // This is probably: Package subpath './package.json' is not defined by "exports"
    }
    let modulePkg;
    let modulePkgFile;
    if (moduleFile !== moduleName) {
        const pairs = [];
        const pkgFile = await (0, exports.findPkgByName)(moduleFile, moduleName, (x, y) => {
            pairs.push([x, y]);
        });
        if (pkgFile && pairs.length > 0) {
            ;
            [rootPkgFile, rootPkg] = pairs[pairs.length - 1];
            if (pairs.length > 1) {
                ;
                [modulePkgFile, modulePkg] = pairs[0];
            }
        }
    }
    return { rootPkgFile, rootPkg, modulePkgFile, modulePkg };
};
exports.findPkgs = findPkgs;
