import Debug from 'debug';
import * as esbuild from 'esbuild';
import { ImportInfo } from './parse';
import { Pkg } from './findPkg';
export declare const hasFile: (file: string) => Promise<boolean>;
export declare const pick: <T>(obj: T, keys: (keyof T)[]) => T;
declare type StatsOpt = boolean | 'tree' | 'table';
declare type BundleResult = {
    size: number;
    zippedSize: number;
    human: {
        size: string;
        zippedSize: string;
    };
    pkg?: Pkg;
    pkgFile?: string;
    modulePkg?: Pkg;
    modulePkgFile?: string;
    stats?: string;
};
export declare type MeasureResult = {
    importInfo: ImportInfo;
    result?: BundleResult;
    error?: Error | unknown;
};
declare type MeasureOptions = {
    /**
     * whether to enable debug log
     * @default false
     */
    debug?: boolean;
    /**
     * the format of stats
     */
    stats?: StatsOpt;
    /**
     * whether to cache bundle result
     * @default false
     */
    cache?: boolean;
    /**
     * custom log function
     */
    log?: Debug.Debugger['log'];
    /**
     * the path of the workspace folder
     */
    workspaceFolder?: string;
    /**
     * the files pattern to strip flow types
     */
    flowPattern?: RegExp;
    /**
     * custom loader for esbuild
     */
    loader?: esbuild.BuildOptions['loader'];
};
/**
 * Measure the bundle size of each import statement
 */
export declare function measureIterable(input: string, fileName?: string | null, { debug, log, stats, cache, workspaceFolder, flowPattern, loader, }?: MeasureOptions): AsyncGenerator<MeasureResult>;
/**
 * Measure the bundle size of each import statement
 */
export declare const measure: (input: string, fileName?: string | null, opts?: MeasureOptions) => Promise<MeasureResult[]>;
export {};
