"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.measure = exports.measureIterable = exports.pick = exports.hasFile = void 0;
const fs_1 = require("fs");
const path_1 = __importDefault(require("path"));
const bytes_1 = __importDefault(require("bytes"));
const debug_1 = __importDefault(require("debug"));
const util_1 = require("util");
const zlib_1 = __importDefault(require("zlib"));
const esbuild = __importStar(require("esbuild"));
const module_1 = require("module");
const parse_1 = require("./parse");
const analyzeMetafile_1 = __importDefault(require("./analyzeMetafile"));
const findPkg_1 = require("./findPkg");
const logger = (0, debug_1.default)('measure-bundle-size');
const gzip = (0, util_1.promisify)(zlib_1.default.gzip);
const gzipSize = (buf) => gzip(buf).then((x) => x.length);
const hasFile = async (file) => Boolean(await fs_1.promises.stat(file).catch(() => null));
exports.hasFile = hasFile;
const pick = (obj, keys) => keys.reduce((acc, k) => ((acc[k] = obj[k]), acc), {});
exports.pick = pick;
const reBuiltin = RegExp(`^node:|^(${module_1.builtinModules.join('|')})(/|$)`);
/** ignore nested core modules  */
const builtinExternalPlugin = {
    name: 'external',
    setup(build) {
        build.onResolve({ filter: reBuiltin, namespace: 'file' }, (args) => ({
            path: args.path,
            external: true,
        }));
    },
};
const reNonRelative = /^[a-z@]/;
/** ignore nested peer deps  */
const peerExternalPlugin = {
    name: 'external',
    setup(build) {
        const pkgCache = new Map();
        const externalsMap = new Map();
        build.onResolve({ filter: reNonRelative, namespace: 'file' }, async (args) => {
            if (reBuiltin.test(args.path)) {
                return null;
            }
            // should passed by esbuild: https://github.com/evanw/esbuild/issues/978
            let pkg;
            const cacheKey = args.resolveDir;
            if (pkgCache.has(cacheKey)) {
                pkg = (await pkgCache.get(cacheKey));
            }
            else {
                const result = (0, findPkg_1.findPkg)(cacheKey).then((pkgFile) => 
                // eslint-disable-next-line @typescript-eslint/no-var-requires
                pkgFile ? require(pkgFile) : null);
                pkgCache.set(cacheKey, result);
                pkg = await result;
            }
            if ((pkg === null || pkg === void 0 ? void 0 : pkg.peerDependencies) && args.path in pkg.peerDependencies) {
                let externals = externalsMap.get(pkg.name);
                if (!externals) {
                    externals = new Set();
                    externalsMap.set(pkg.name, externals);
                }
                externals.add(args.path);
                return {
                    path: args.path,
                    external: true,
                };
            }
        });
        build.onEnd(() => {
            if (externalsMap.size) {
                logger(`peer externals`, externalsMap);
            }
        });
    },
};
const stripFlowTypesPlugin = (pattern = /\/node_modules\/(@react-native|react-native|react-native-linear-gradient)\/(.*)\.js$/) => {
    return {
        name: 'flow',
        setup(build) {
            build.onLoad({ filter: pattern }, async (args) => {
                const source = await fs_1.promises.readFile(args.path, 'utf8');
                const { stripFlowTypes } = await Promise.resolve().then(() => __importStar(require('./stripFlowTypes')));
                const output = stripFlowTypes(source.toString(), args.path);
                const contents = output !== null && output !== void 0 ? output : '';
                return {
                    contents,
                    loader: 'jsx',
                };
            });
        },
    };
};
const bundleCache = new Map();
const genCacheKey = (pkg, names) => `${pkg.name}:${pkg.version}:${names}`;
const timeMark = () => {
    const startTimes = new Map();
    const values = new Map();
    return {
        start(label) {
            startTimes.set(label, Date.now());
        },
        end(label) {
            const startTime = startTimes.get(label);
            if (!startTime) {
                throw new Error(`No label: ${label}`);
            }
            values.set(label, Date.now() - startTime);
        },
        print() {
            return [...values].map(([k, v]) => `${k} in ${v}ms`).join(', ');
        },
    };
};
const pickPkgFields = (pkg) => {
    return (0, exports.pick)(pkg, [
        'name',
        'version',
        'description',
        'homepage',
        'dependencies',
        'peerDependencies',
    ]);
};
/**
 * TODO: add more support for RN
 * - add a platform/target option: `target: 'web' | 'node' | 'neutral' | 'react-native'`
 * - add exports support: `conditions: ['react-native', 'import']`
 * - use jsx loader for .js
 */
const bundle = async (statement, importInfo, { baseDir, projectPkgFile, flowPattern, loader, stats: statsOpt = false, cache: cacheOpt = false, }) => {
    const bundleMark = timeMark();
    const modulePath = importInfo.from;
    const [part1] = modulePath.split('/');
    if (!/^[a-z@]/.test(part1)) {
        // file path, alias
        throw new Error('Skip non-npm packages');
    }
    if (reBuiltin.test(part1)) {
        throw new Error('Skip builtin modules');
    }
    const entryInput = `${statement}\n${(0, parse_1.exportImported)(importInfo)}`;
    // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
    const { rootPkgFile, rootPkg, modulePkgFile, modulePkg } = await (0, findPkg_1.findPkgs)(modulePath, baseDir).catch(() => {
        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-unsafe-return
        return {};
    });
    // TODO: use current package path as the cache key
    const cacheKey = cacheOpt && rootPkg !== undefined && genCacheKey(rootPkg, entryInput);
    if (cacheOpt && cacheKey) {
        const cache = bundleCache.get(cacheKey);
        if (cache) {
            logger('using cache:', importInfo.from);
            return cache;
        }
    }
    bundleMark.start('bundle');
    // TODO: alert indirect deps, eg.: react-redux > react-dom
    // exclude peer deps, eg: react-router-dom > react
    const external = rootPkg ? Object.keys({ ...rootPkg.peerDependencies }) : [];
    // reduce depth of relative path in `metafile`, can't resolve directly to `node_modules` because it may not exist (or in monorepo)
    const workingDir = projectPkgFile ? path_1.default.dirname(projectPkgFile) : baseDir;
    /**
     * - build in memory, no file writing
     * - respect 'browser' field in package.json
     * - production build, respect `NODE_ENV`
     *
     * @see https://esbuild.github.io/api/#build-api
     */
    const buildResult = await esbuild.build({
        stdin: {
            loader: 'ts',
            contents: entryInput,
            resolveDir: workingDir,
            sourcefile: '<import>',
        },
        outfile: '<bundle>.js',
        absWorkingDir: workingDir,
        plugins: [
            builtinExternalPlugin,
            peerExternalPlugin,
            stripFlowTypesPlugin(flowPattern),
        ],
        resolveExtensions: [
            // RN
            '.ios.js',
            '.android.js',
            '.native.js',
            // defaults to .jsx,.js,.tsx,.ts,.css,.json
            '.jsx',
            '.js',
            '.tsx',
            '.ts',
            '.css',
            '.json',
        ],
        loader: {
            '.node': 'binary',
            // RN or web app may use static assets
            '.jpeg': 'empty',
            '.jpg': 'empty',
            '.png': 'empty',
            '.webp': 'empty',
            '.gif': 'empty',
            ...loader,
        },
        external,
        target: 'esnext',
        format: 'esm',
        platform: 'browser',
        metafile: statsOpt !== false,
        minify: true,
        bundle: true,
        write: false,
    });
    bundleMark.end('bundle');
    // analyze on hover?
    let stats;
    if (buildResult.metafile) {
        bundleMark.start('analyze');
        if (statsOpt === 'tree') {
            stats = await esbuild.analyzeMetafile(buildResult.metafile);
        }
        else if (statsOpt === 'table') {
            stats = (0, analyzeMetafile_1.default)(buildResult.metafile);
        }
        bundleMark.end('analyze');
        // if (stats) {
        //   log('stats:\n' + stats)
        // }
    }
    // concat <bundle>.js, <bundle>.css, etc
    bundleMark.start('zip');
    const sizes = await Promise.all(buildResult.outputFiles.map(async (file) => {
        const itsSize = file.contents.length;
        return [
            itsSize,
            itsSize === 0 ? 0 : await gzipSize(Buffer.from(file.contents)),
        ];
    }));
    bundleMark.end('zip');
    let size = 0;
    let zippedSize = 0;
    for (const [s1, s2] of sizes) {
        size += s1;
        zippedSize += s2;
    }
    const human = { size: (0, bytes_1.default)(size), zippedSize: (0, bytes_1.default)(zippedSize) };
    logger(`${bundleMark.print()}, ${statement}, ${human.size}`);
    const toRelative = path_1.default.relative.bind(null, process.cwd());
    const result = {
        size,
        zippedSize,
        human,
        pkg: rootPkg ? pickPkgFields(rootPkg) : undefined,
        pkgFile: rootPkgFile ? toRelative(rootPkgFile) : undefined,
        modulePkg: modulePkg ? pickPkgFields(modulePkg) : undefined,
        modulePkgFile: modulePkgFile ? toRelative(modulePkgFile) : undefined,
        stats,
    };
    if (cacheOpt && cacheKey) {
        bundleCache.set(cacheKey, result);
    }
    return result;
};
const withValue = async (valueFn) => {
    try {
        const r = await valueFn();
        return [void 0, r];
    }
    catch (err) {
        return [err, void 0];
    }
};
/**
 * Measure the bundle size of each import statement
 */
async function* measureIterable(input, fileName, { debug, log, stats, cache, workspaceFolder, flowPattern, loader, } = {}) {
    if (debug) {
        logger.enabled = true;
    }
    if (log) {
        logger.log = log;
    }
    logger('process', fileName);
    const baseDir = fileName && (await (0, exports.hasFile)(fileName))
        ? path_1.default.dirname(fileName)
        : workspaceFolder && (await (0, exports.hasFile)(workspaceFolder))
            ? workspaceFolder
            : null;
    if (!baseDir) {
        throw new Error('Can not resolve `fileName` or `workspaceFolder`');
    }
    const projectPkgFile = await (0, findPkg_1.findPkg)(baseDir);
    const parseMark = timeMark();
    parseMark.start('parse');
    const result = (0, parse_1.parse)(input);
    parseMark.end('parse');
    logger(parseMark.print());
    const bundleOpts = {
        baseDir,
        projectPkgFile,
        stats,
        cache,
        flowPattern,
        loader,
    };
    for (const importInfo of result.imports) {
        const statement = input.substring(importInfo.start, importInfo.end);
        // await new Promise((resolve) => setTimeout(resolve, 500))
        const [error, result] = await withValue(() => bundle(statement, importInfo, bundleOpts));
        yield { importInfo, result, error };
    }
}
exports.measureIterable = measureIterable;
/**
 * Measure the bundle size of each import statement
 */
const measure = async (input, fileName, opts = {}) => {
    const results = [];
    for await (const result of measureIterable(input, fileName, opts)) {
        results.push(result);
    }
    return results;
};
exports.measure = measure;
