"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.exportImported = exports.parse = void 0;
const parser = __importStar(require("@babel/parser"));
const traverse_1 = __importDefault(require("@babel/traverse"));
const parse = (input) => {
    const ast = parser.parse(input, {
        sourceType: 'module',
        plugins: [
            'jsx',
            'typescript',
            // https://babeljs.io/docs/babel-plugin-proposal-decorators
            ['decorators', {}],
        ],
    });
    if (ast.errors.length) {
        throw new Error(ast.errors[0].code);
    }
    const format = (x) => {
        const { start, end, specifiers, source } = x;
        const info = {
            start: start,
            end: end,
            from: source.value,
        };
        for (const s of specifiers) {
            if (s.type === 'ImportNamespaceSpecifier') {
                info.namespace = { name: s.local.name, usingProps: [] };
            }
            else if (s.type === 'ImportDefaultSpecifier') {
                info.names || (info.names = {});
                info.names.default = s.local.name;
            }
            else if (s.type === 'ImportSpecifier') {
                info.names || (info.names = {});
                info.names[s.imported.type === 'StringLiteral'
                    ? s.imported.value
                    : s.imported.name] = s.local.name;
            }
        }
        return info;
    };
    // top level only
    const imports = ast.program.body
        .filter((x) => x.type === 'ImportDeclaration')
        .map((x) => format(x));
    const namespaces = imports
        .map((x) => x.namespace)
        .filter(Boolean);
    if (namespaces.length) {
        const names = new Set(namespaces.map((x) => x.name));
        const createVisitor = (type, propType) => (path) => {
            const { name } = path.node;
            if (!names.has(name)) {
                return;
            }
            const { parent } = path;
            if (parent.type === type) {
                const binding = path.scope.getBinding(name);
                if (binding &&
                    binding.path.node.type === 'ImportNamespaceSpecifier') {
                    const namespace = namespaces.find((e) => e.name === name);
                    // ignore Expression | PrivateName
                    if (parent.property.type === propType) {
                        namespace.usingProps.push({
                            start: parent.start,
                            end: parent.end,
                            name: parent.property.name,
                        });
                    }
                }
            }
        };
        (0, traverse_1.default)(ast, {
            Identifier: createVisitor('MemberExpression', 'Identifier'),
            JSXIdentifier: createVisitor('JSXMemberExpression', 'JSXIdentifier'),
        });
    }
    return { imports };
};
exports.parse = parse;
const exportImported = (info) => {
    const { names, namespace } = info;
    return [
        names && `export {${Object.values({ ...names }).join(', ')}}`,
        (namespace === null || namespace === void 0 ? void 0 : namespace.usingProps.length) &&
            `export default [${namespace.usingProps
                .map((x) => `${namespace.name}.${x.name}`)
                .join(', ')}]`,
    ]
        .filter(Boolean)
        .join('\n');
};
exports.exportImported = exportImported;
